#include "ec.h"

static void ec_mul2(const EC *ec, ECP *R, const ECP *P) {
	mpz_t t0, t1, t2;

	mpz_init2(t0, mp_bits_per_limb*(2*mpz_size(ec->p)+2));
	mpz_init2(t1, mp_bits_per_limb*(2*mpz_size(ec->p)+2));
	mpz_init2(t2, mp_bits_per_limb*(2*mpz_size(ec->p)+2));

	mpz_add(t0, P->y, P->y);
	mpz_invert(t0, t0, ec->p);
	mpz_pow_ui(t1, P->x, 2);
	mpz_mul_ui(t1, t1, 3);
	mpz_add(t1, t1, ec->a);
	mpz_mod(t1, t1, ec->p);
	mpz_mul(t1, t1, t0);
	mpz_mod(t1, t1, ec->p);

	mpz_add(t0, P->x, P->x);
	mpz_mul(t2, t1, t1);
	mpz_sub(t2, t2, t0);
	mpz_mod(t2, t2, ec->p);

	mpz_sub(t0, P->x, t2);
	mpz_mul(t0, t0, t1);
	mpz_sub(t0, t0, P->y);
	mpz_set(R->x, t2);
	mpz_mod(R->y, t0, ec->p);

	mpz_clear(t0);
	mpz_clear(t1);
	mpz_clear(t2);
}

void ec_add(const EC *ec, ECP *R, const ECP *P, const ECP *Q) {
	mpz_t t0, t1, t2;

	if(mpz_cmp(P->x, Q->x) == 0 && mpz_cmp(P->y, Q->y) == 0) {
		ec_mul2(ec, R, P);
		return;
	}

	mpz_init2(t0, mp_bits_per_limb*(2*mpz_size(ec->p)+2));
	mpz_init2(t1, mp_bits_per_limb*(2*mpz_size(ec->p)+2));
	mpz_init2(t2, mp_bits_per_limb*(2*mpz_size(ec->p)+2));

	mpz_sub(t0, Q->y, P->y);
	mpz_sub(t1, Q->x, P->x);
	mpz_invert(t1, t1, ec->p);
	mpz_mul(t1, t0, t1);
	mpz_mod(t1, t1, ec->p);

	mpz_pow_ui(t0, t1, 2);
	mpz_sub(t0, t0, P->x);
	mpz_sub(t0, t0, Q->x);
	mpz_mod(t0, t0, ec->p);

	mpz_sub(t2, P->x, t0);
	mpz_mul(t2, t2, t1);
	mpz_sub(t2, t2, P->y);
	mpz_set(R->x, t0);
	mpz_mod(R->y, t2, ec->p);

	mpz_clear(t0);
	mpz_clear(t1);
	mpz_clear(t2);
}

void ec_mul(const EC *ec, ECP *R, const ECP *P, mpz_t k) {
	int i, first = 1;
	ECP Q;

	mpz_init_set(Q.x, P->x);
	mpz_init_set(Q.y, P->y);

	for(i=0; i<mpz_sizeinbase(k,2)-1; i++) {
		if(mpz_tstbit(k, i)) {
			if(first) {
				mpz_set(R->x, Q.x);
				mpz_set(R->y, Q.y);
				first = 0;
			} else {
				ec_add(ec, R, R, &Q);
			}
		}
		ec_mul2(ec, &Q, &Q);
	}
	if(first) {
		mpz_set(R->x, Q.x);
		mpz_set(R->y, Q.y);
		first = 0;
	} else {
		ec_add(ec, R, R, &Q);
	}

	mpz_clear(Q.x);
	mpz_clear(Q.y);
}

