#include <stdio.h>
#include "ec.h"
#include "ecdsa.h"

int main(void) {
	EC ec;
	ECP Q;
	mpz_t d, e, r, s;
	gmp_randstate_t state;

	mpz_init_set_ui(ec.a, 0);
	mpz_init_set_ui(ec.b, 7);
	mpz_init_set_str(ec.p,
	"fffffffffffffffffffffffffffffffffffffffffffffffffffffffefffffc2f", 16);
	mpz_init_set_str(ec.G.x,
	"79be667ef9dcbbac55a06295ce870b07029bfcdb2dce28d959f2815b16f81798", 16);
	mpz_init_set_str(ec.G.y,
	"483ada7726a3c4655da4fbfc0e1108a8fd17b448a68554199c47d08ffb10d4b8", 16);
	mpz_init_set_str(ec.n,
	"fffffffffffffffffffffffffffffffebaaedce6af48a03bbfd25e8cd0364141", 16);

	gmp_randinit_default(state);
	gmp_randseed_ui(state, 0xdeadbeef);

	mpz_init(r);
	mpz_init(s);

	mpz_init_set_ui(d, 0x123);
	mpz_init_set_ui(e, 0xabc);

	mpz_init(Q.x);
	mpz_init(Q.y);
	ec_mul(&ec, &Q, &ec.G, d);

	ecdsa_sign(&ec, state, d, e, r, s);
	ecdsa_verify(&ec, &Q, e, r, s);

	gmp_printf("r = %Zx\n", r);
	gmp_printf("s = %Zx\n", s);

	if(ecdsa_verify(&ec, &Q, e, r, s)) {
		printf("Verified!\n");
	} else {
		printf("NOT verified!\n");
	}

	return 0;
}

